@@ -23,8 +23,6 @@ Contact: yuvraaj@gmail.com
 #include "GvXMLParser.h"
 #include "MyXmlErrorHandler.h"
 
-#include <QtXmlPatterns>
-
 GVApi::GVApi(bool bEmitLog, QObject *parent)
 : QObject(parent)
 , emitLog(bEmitLog)
@@ -32,6 +30,7 @@ GVApi::GVApi(bool bEmitLog, QObject *parent)
 , nwMgr(this)
 , jar(new CookieJar(NULL))
 , dbgAlwaysFailDialing (false)
+, scriptEngine (this)
 {
     nwMgr.setCookieJar (jar);
 }//GVApi::GVApi
@@ -712,7 +711,7 @@ GVApi::onLogin2(bool success, const QByteArray &response, QNetworkReply *reply,
 
         // After this we should have completed login. Check for cookie "gvx"
         foreach (QNetworkCookie cookie, jar->getAllCookies ()) {
-            if (cookie.name () == "gvx") {
+            if (cookie.name() == "gvx") {
                 loggedIn = true;
                 break;
             }
@@ -926,7 +925,7 @@ GVApi::getRnr(AsyncTaskToken *token)
 {
     Q_DEBUG("User authenticated, now looking for RNR.");
 
-    bool rv = doGet("https://www.google.com/voice/m/i/all", token, this,
+    bool rv = doGet(GV_HTTPS_M "/i/all", token, this,
                     SLOT (onGotRnr(bool, const QByteArray &,
                                    QNetworkReply *, void *)));
     Q_ASSERT(rv);
@@ -935,7 +934,7 @@ GVApi::getRnr(AsyncTaskToken *token)
 }//GVApi::getRnr
 
 void
-GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
+GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *reply,
                 void *ctx)
 {
     AsyncTaskToken *token = (AsyncTaskToken *)ctx;
@@ -944,6 +943,44 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
     do { // Begin cleanup block (not a loop)
         if (!success) break;
 
+        QString strReplyUrl = reply->url().toString();
+        if (!strReplyUrl.contains ("/i/all")) {
+            success = false;
+
+            if (token->outParams["attempts"].toInt() != 0) {
+                Q_WARN("Too many attempts at relogin");
+                break;
+            }
+
+            Q_DEBUG(strReplyUrl);
+
+            // Probably failed to login correctly. Try one more time.
+            AsyncTaskToken *internalLogoutTask = new AsyncTaskToken(this);
+            if (!internalLogoutTask) {
+                Q_WARN("Failed to login because failed to logout!!");
+                break;
+            }
+
+            internalLogoutTask->callerCtx = token;
+
+            success =
+            connect(internalLogoutTask, SIGNAL(completed(AsyncTaskToken*)),
+                    this,  SLOT(internalLogoutForReLogin(AsyncTaskToken*)));
+            Q_ASSERT(success);
+
+            success = logout(internalLogoutTask);
+            if (!success) {
+                internalLogoutTask->deleteLater ();
+                break;
+            }
+
+            int attempts = 1;
+            token->outParams["attempts"] = attempts;
+
+            success = true;
+            break;
+        }
+
         success = false;
         int pos = strResponse.indexOf ("_rnr_se");
         if (pos == -1) {
@@ -982,6 +1019,14 @@ GVApi::onGotRnr(bool success, const QByteArray &response, QNetworkReply *,
     }
 }//GVApi::onGotRnr
 
+void
+GVApi::internalLogoutForReLogin(AsyncTaskToken *token)
+{
+    AsyncTaskToken *origToken = (AsyncTaskToken *) token->callerCtx;
+    login (origToken);
+    token->deleteLater ();
+}//GVApi::internalLogoutForReLogin
+
 bool
 GVApi::logout(AsyncTaskToken *token)
 {
@@ -1062,7 +1107,6 @@ GVApi::onGetPhones(bool success, const QByteArray &response, QNetworkReply *,
         simpleReader.parse (&inputSource, false);
 
         QString strTemp;
-        QScriptEngine scriptEngine;
         strTemp = "var obj = " + xmlHandler.strJson;
         scriptEngine.evaluate (strTemp);
         if (scriptEngine.hasUncaughtException ()) {
@@ -1325,7 +1369,6 @@ GVApi::parseInboxJson(const QString &strJson, const QString &strHtml,
 
     do { // Begin cleanup block (not a loop)
         QString strTemp;
-        QScriptEngine scriptEngine;
         strTemp = "var obj = " + strJson;
         scriptEngine.evaluate (strTemp);
         if (scriptEngine.hasUncaughtException ()) {
@@ -1631,12 +1674,15 @@ GVApi::callOut(AsyncTaskToken *token)
         return true;
     }
 
+    QString dest = token->inParams["destination"].toString();
     QUrl url(GV_HTTPS_M "/x");
     url.addQueryItem("m" , "call");
-    url.addQueryItem("n" , token->inParams["destination"].toString());
+    url.addQueryItem("n" , dest);
     url.addQueryItem("f" , "");
     url.addQueryItem("v" , "7");
 
+    Q_DEBUG(QString("Call back: dest=%1, using=").arg(dest));
+
     QByteArray content;
     QList<QNetworkCookie> allCookies = jar->getAllCookies ();
     foreach (QNetworkCookie cookie, allCookies) {
@@ -1682,7 +1728,6 @@ GVApi::onCallout(bool success, const QByteArray &response, QNetworkReply *,
         Q_DEBUG(strTemp);
 #endif
 
-        QScriptEngine scriptEngine(this);
         strTemp = QString("var obj = %1; "
                           "obj.call_through_response.access_number;")
                           .arg(strTemp);
@@ -1756,6 +1801,8 @@ GVApi::callBack(AsyncTaskToken *token)
     strContent += QString("&subscriberNumber=%1&remember=1&_rnr_se=%2")
                     .arg (strSelfNumber, rnr_se);
 
+    Q_DEBUG(QString("Call back request = %1").arg(strContent));
+
     bool rv = doPostForm(url, strContent.toAscii (), token, this,
                          SLOT(onCallback(bool, const QByteArray &,
                                          QNetworkReply *, void *)));
@@ -1787,7 +1834,6 @@ GVApi::onCallback(bool success, const QByteArray &response, QNetworkReply *,
             strTemp = strTemp.mid (strTemp.indexOf ('{'));
         }
 
-        QScriptEngine scriptEngine(this);
         strTemp = QString("var obj = %1; obj.ok;").arg(strTemp);
         strTemp = scriptEngine.evaluate (strTemp).toString ();
         if (scriptEngine.hasUncaughtException ()) {
@@ -1897,7 +1943,6 @@ GVApi::onSendSms(bool success, const QByteArray &response, QNetworkReply *,
             strTemp = strTemp.mid (strTemp.indexOf ('{'));
         }
 
-        QScriptEngine scriptEngine(this);
         strTemp = QString("var obj = %1; obj.send_sms_response.status.status;")
                     .arg(strTemp);
         strTemp = scriptEngine.evaluate (strTemp).toString ();
@@ -2056,7 +2101,6 @@ GVApi::onMarkAsRead(bool success, const QByteArray &response, QNetworkReply *,
             strTemp = strTemp.mid (strTemp.indexOf ('{'));
         }
 
-        QScriptEngine scriptEngine(this);
         strTemp = QString("var obj = %1; obj.ok;").arg(strTemp);
         strTemp = scriptEngine.evaluate (strTemp).toString ();
         if (scriptEngine.hasUncaughtException ()) {
